<?php

namespace App\Http\Controllers;

use Laravel\Lumen\Routing\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Helpers\AccessLevelHelper;
use \Illuminate\Support\Facades\Lang;
use Illuminate\Support\Facades\File;

class ApplicantsController extends Controller
{
  private $request;
  private $user_sciper;
  private $user_permissions;
  private $user_role;

  public function __construct(Request $request)
  {
    $this->request = $request;
    $this->user_sciper = $this->request->get('user_sciper');
    $this->user_permissions = $this->request->get('user_permissions');
    $this->user_role = $this->request->get('user_role');
  }

  public function getAll()
  {
    $applicants = [];
    foreach ($this->user_permissions as $job) {
      $job_applicants = DB::table('applicant')->where('applicant_formation', $job)->get();
      if (count($job_applicants)) {
        array_push($applicants, $job_applicants);
      }
    }
    return $applicants[0];
  }

  public function getJobApplicants($job)
  {
    if (AccessLevelHelper::isJobValid($job)) {
      $has_access = AccessLevelHelper::hasAccessToJob($job, $this->user_permissions);
      if ($has_access) {
        return DB::table('applicant')->where('applicant_formation', $job)->get();
      } else {
        return abort(403, lang::get('http.unauthorized'));
      }
    } else {
      return abort(404, lang::get('http.notfound'));
    }
  }

  public function getOneById($id)
  {
    $applicant_job = DB::table('applicant')->where('applicant_id', $id)->pluck('applicant_formation')->first();
    $has_access = AccessLevelHelper::hasAccessToJob($applicant_job, $this->user_permissions);
    if ($has_access) {
      $data = $this->getOne($id);
      return response()->json($data, 200);
    } else {
      return abort(403, lang::get('http.unauthorized'));
    }
  }

  public function updateStatus($id)
  {
    $applicant_job = DB::table('applicant')->where('applicant_id', $id)->pluck('applicant_formation')->first();
    $has_access = AccessLevelHelper::hasAccessToJob($applicant_job, $this->user_permissions);
    $has_permitted_role = AccessLevelHelper::hasPermittedRole($this->user_role, 'responsable');
    if ($has_access && $has_permitted_role) {
      $this->validate($this->request, [
        'status' => 'required'
      ], [lang::get('validation.required')]);

      $new_status = $this->request->input('status');
      return DB::table('applicant')->where('applicant_id', $id)->update(['applicant_application_status' => $new_status]);
    } else {
      return abort(403, lang::get('http.unauthorized'));
    }
  }

  public function delete($id)
  {
    $applicant_job = DB::table('applicant')->where('applicant_id', $id)->pluck('applicant_formation')->first();
    $has_access = AccessLevelHelper::hasAccessToJob($applicant_job, $this->user_permissions);

    $has_permitted_role = AccessLevelHelper::hasPermittedRole($this->user_role, 'responsable');
    if ($has_access && $has_permitted_role) {
      return DB::table('applicant')->where('applicant_id', $id)->delete();
    } else {
      return abort(403, lang::get('http.unauthorized'));
    }
  }

  public function export($id)
  {
    $applicant_job = DB::table('applicant')->where('applicant_id', $id)->pluck('applicant_formation')->first();
    $has_access = AccessLevelHelper::hasAccessToJob($applicant_job, $this->user_permissions);
    if ($has_access) {
      $tmp_file_path = sys_get_temp_dir() . '\\' .$id .'-export.json';
      $json = json_encode($this->getOne($id));
      File::put($tmp_file_path, $json);
      return response()->download($tmp_file_path, $id .'-export.json', ['Content-Type' => 'application/json'])->deleteFileAfterSend(true);
    } else {
      return abort(403, lang::get('http.unauthorized'));
    }
  }

  private function getOne($id)
  {
    // get applicant
    $applicant = DB::table('applicant')->where('applicant_id', $id)->first();
    // get responsibles
    $main_resp = DB::table('applicant')->where('applicant_id', $id)
      ->select('responsible_id', 'responsible_gender', 'responsible_name', 'responsible_fsname', 'responsible_street', 'responsible_npa', 'responsible_phone')
      ->join('responsible', 'applicant.fk_applicant_main_responsible', '=', 'responsible.responsible_id')
      ->first();

    $sec_resp = DB::table('applicant')->where('applicant_id', $id)
      ->select('responsible_id', 'responsible_gender', 'responsible_name', 'responsible_fsname', 'responsible_street', 'responsible_npa', 'responsible_phone')
      ->join('responsible', 'applicant.fk_applicant_sec_responsible', '=', 'responsible.responsible_id')
      ->first();

    // get scolarity
    $scolarities = DB::table('scolarity')->where('fk_applicant_id', $id)->get();

    // get pro activities
    $pro_activities = DB::table('professional_activity')->where('fk_applicant_id', $id)->get();

    // get trainings
    $trainings = DB::table('training')->where('fk_applicant_id', $id)->get();

    // get files (infos)
    $files = DB::table('file')->select('file_id', 'file_name')->where('fk_applicant_id', $id)->get();

    return [
      "personal_data" => $applicant,
      "responsibles" => [
        "main" => $main_resp,
        "secondary" => $sec_resp
      ],
      "scolarities" => $scolarities,
      "pro_activities" => $pro_activities,
      "training" => $trainings,
      "files" => $files
    ];
  }
}
