<?php

namespace App\Http\Controllers;

use Laravel\Lumen\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Http\Request;
use App\Helpers\AccessLevelHelper;
use \Illuminate\Support\Facades\Lang;


class MarkersController extends Controller
{
  private $request;
  private $user_sciper;
  private $user_permissions;

  public function __construct(Request $request)
  {
    $this->request = $request;
    $this->user_sciper = $this->request->get('user_sciper');
    $this->user_permissions = $this->request->get('user_permissions');
  }

  public function getApplicantMarkers($id)
  {
    $applicant_job = DB::table('applicant')->where('applicant_id', $id)->pluck('applicant_formation')->first();
    $has_access = AccessLevelHelper::hasAccessToJob($applicant_job, $this->user_permissions);
    if ($has_access) {
      return DB::table('marker')->where('fk_applicant_id', $id)->get();
    } else {
      return abort(403, lang::get('http.unauthorized'));
    }
  }

  public function getUserMarkerOnApplicant($id)
  {
    return DB::table('marker')->where('fk_applicant_id', $id)->where('marker_owner_sciper', $this->user_sciper)->get();
  }

  public function create()
  {
    $this->validate($this->request, [
      'type' => 'required',
      'applicant_id' => 'required',
    ], [lang::get('validation.required')]);

    $new_type = $this->request->input('type');
    $new_applicant_id = $this->request->input('applicant_id');

    $applicant_job = DB::table('applicant')->where('applicant_id', $new_applicant_id)->pluck('applicant_formation')->first();
    $has_access = AccessLevelHelper::hasAccessToJob($applicant_job, $this->user_permissions);

    if ($has_access) {
      $inserted_id = DB::table('marker')->insertGetId([
        "marker_owner_sciper" => $this->user_sciper,
        "marker_type" => $new_type,
        "fk_applicant_id" => $new_applicant_id
      ]);
      return ["message" => lang::get('http.success.created.marker'), "id" => $inserted_id];
    } else {
      return abort(403, lang::get('http.unauthorized'));
    }
  }

  public function update($id)
  {
    $this->validate($this->request, [
      'type' => 'required'
    ], [lang::get('validation.required')]);

    $new_type = $this->request->input('type');
    $wanted_marker_exists = DB::table('marker')->where('marker_id', $id)->where('marker_owner_sciper', $this->user_sciper)->exists();
    if ($wanted_marker_exists) {
      DB::table('marker')->where('marker_id', $id)->update(['marker_type' => $new_type]);
      return ["message" => lang::get('http.success.updated.marker'), "id" => $id];
    } else {
      return abort(403, lang::get('http.unauthorized'));
    }
  }

  public function delete($id)
  {
    $wanted_marker_exists = DB::table('marker')->where('marker_id', $id)->where('marker_owner_sciper', $this->user_sciper)->exists();
    if ($wanted_marker_exists) {
      DB::table('marker')->where('marker_id', $id)->where('marker_owner_sciper', $this->user_sciper)->delete();
      return ["message" => lang::get('http.success.deleted.marker')];
    } else {
      return abort(403, lang::get('http.unauthorized'));
    }
  }

}
